--[[
	Copyright (C) Udorn (Blackhand)
--]]

petbm = petbm or {}
   
petbm.PetStable = petbm.PetBattleMaster:NewModule("PetStable", "AceEvent-3.0")
local L = petbm.Locale.GetInstance()

local log = petbm.Debug:new("PetStable")

local IS_WILD = false -- WTF why is it false ?!

local function _RestoreFilter(self)
	-- search text
	C_PetJournal.SetSearchFilter(self.searchText or "")

	-- pet types
	local petTypes = self.petTypes
	local numTypes = C_PetJournal.GetNumPetTypes()
	for i=1,numTypes do
		C_PetJournal.SetPetTypeFilter(i, petTypes[i])
	end
	
	-- pet sources
	local petSources = self.petSources
	local numSources = C_PetJournal.GetNumPetSources()
	for i=1,numSources do
		C_PetJournal.SetPetSourceFilter(i, petSources[i])
	end
	
	-- flags
	local flags = self.flags
	C_PetJournal.SetFlagFilter(LE_PET_JOURNAL_FLAG_COLLECTED, flags[LE_PET_JOURNAL_FLAG_COLLECTED])
	C_PetJournal.SetFlagFilter(LE_PET_JOURNAL_FLAG_FAVORITES, flags[LE_PET_JOURNAL_FLAG_FAVORITES])
	C_PetJournal.SetFlagFilter(LE_PET_JOURNAL_FLAG_NOT_COLLECTED, flags[LE_PET_JOURNAL_FLAG_NOT_COLLECTED])
end

local function _ClearFilter(self)
	-- search text
	self.searchText = petbm.PetBattleMaster.searchText
	
	-- pet types
	local petTypes = wipe(self.petTypes)
	local numTypes = C_PetJournal.GetNumPetTypes()
	for i=1,numTypes do
		petTypes[i] = not C_PetJournal.IsPetTypeFiltered(i)
	end
	
	-- pet sources
	local petSources = wipe(self.petSources)
	local numSources = C_PetJournal.GetNumPetSources()
	for i=1,numSources do
		petSources[i] = not C_PetJournal.IsPetSourceFiltered(i)
	end
	
	-- flags
	local flags = wipe(self.flags)
	flags[LE_PET_JOURNAL_FLAG_COLLECTED] = not C_PetJournal.IsFlagFiltered(LE_PET_JOURNAL_FLAG_COLLECTED)
	flags[LE_PET_JOURNAL_FLAG_FAVORITES] = not C_PetJournal.IsFlagFiltered(LE_PET_JOURNAL_FLAG_FAVORITES)
	flags[LE_PET_JOURNAL_FLAG_NOT_COLLECTED] = not C_PetJournal.IsFlagFiltered(LE_PET_JOURNAL_FLAG_NOT_COLLECTED)
	
	-- clear
	C_PetJournal.SetSearchFilter("")
	C_PetJournal.AddAllPetTypesFilter()
	C_PetJournal.AddAllPetSourcesFilter()
	C_PetJournal.SetFlagFilter(LE_PET_JOURNAL_FLAG_COLLECTED, true)
	C_PetJournal.SetFlagFilter(LE_PET_JOURNAL_FLAG_FAVORITES, false)
	C_PetJournal.SetFlagFilter(LE_PET_JOURNAL_FLAG_NOT_COLLECTED, true)
end

local function PET_JOURNAL_LIST_UPDATE(self)
	-- delay because multi-events are possible
	if (not self.isUpdating) then
		self.nextUpdate = time() + 1
	end	
end

local function GetOwnPets(self)
	return ipairs(self.pets)
end

local function IsSpeciesIdKnown(self, speciesId)
	return self.speciesIds[speciesId]
end

local function _OnUpdate(frame)

	local self = frame.obj

	if (not self.nextUpdate or (time() < self.nextUpdate)) then
		return
	end
	
	if (self.isUpdating) then
		-- we are triggering the event, so we should leave immediately!
		return
	end
	
	self.nextUpdate = nil
	
	log:Debug("PET_JOURNAL_LIST_UPDATE")
	self.isUpdating = true
	_ClearFilter(self)
	
	local numPets, numOwned = C_PetJournal.GetNumPets(IS_WILD)
	if (numPets and numOwned and numOwned ~= #self.pets) then
		local pets = wipe(self.pets)
		local speciesIds = wipe(self.speciesIds)
		for i = 1,numPets do
			-- why the hell is quality not contained here? :-(
			local petId, speciesId, owned, customName, level, favorite, isRevoked, name, icon, petType, creatureId = C_PetJournal.GetPetInfoByIndex(i, IS_WILD)
			speciesIds[speciesId] = true
			if (owned) then
				local quality = select(5, C_PetJournal.GetPetStats(petId))
				if (not quality) then
					wipe(pets)
					break
				end
				tinsert(pets, {petId = petId, speciesId = speciesId, quality = quality, creatureId = creatureId, level = level})
			end
		end
	end

	_RestoreFilter(self)
	self.isUpdating = nil
	log:Debug("Update exit with [%s] pets and [%s] speciesIds", #self.pets, numPets)
end

local function OnInitialize(self)
	self.pets = {}
	self.rarityCache = {}
	self.petTypes = {}
	self.petSources = {}
	self.flags = {}
	self.speciesIds = {}
	self.timerFrame = CreateFrame("Frame", UIParent)
	self.timerFrame.obj = self
	self.timerFrame:SetScript("OnUpdate", _OnUpdate)
end

local function OnEnable(self)
	log:Debug("OnEnable")
	self:RegisterEvent("PET_JOURNAL_LIST_UPDATE")
	self:RegisterEvent("PET_JOURNAL_PET_DELETED")
end

petbm.PetStable.OnInitialize = OnInitialize
petbm.PetStable.OnEnable = OnEnable
petbm.PetStable.GetOwnPets = GetOwnPets
petbm.PetStable.PET_JOURNAL_LIST_UPDATE = PET_JOURNAL_LIST_UPDATE
petbm.PetStable.PET_JOURNAL_PET_DELETED = PET_JOURNAL_LIST_UPDATE
petbm.PetStable.IsSpeciesIdKnown = IsSpeciesIdKnown